@@ -2,7 +2,7 @@ from .base import GenericDoc, GenericQuery, GenericQrel, GenericScoredDoc, Gener
 from .base import BaseDocs, BaseQueries, BaseQrels, BaseScoredDocs, BaseDocPairs
 from .csv_fmt import CsvDocs, CsvQueries, CsvDocPairs
 from .tsv import TsvDocs, TsvQueries, TsvDocPairs
-from .trec import TrecDocs, TrecQueries, TrecXmlQueries, TrecColonQueries, TrecQrels, TrecPrels, TrecScoredDocs, TrecDoc, TitleUrlTextDoc, TrecQuery, TrecSubtopic, TrecQrel, TrecPrel
+from .trec import TrecDocs, TrecQueries, TrecXmlQueries, TrecColonQueries, TrecQrels, TrecPrels, TrecScoredDocs, TrecDoc, TitleUrlTextDoc, TrecQuery, TrecSubtopic, TrecQrel, TrecPrel, TrecParsedDoc
 from .touche import (
     ToucheQueries, ToucheQuery, ToucheTitleQuery, ToucheQrels
 )
@@ -13,3 +13,5 @@ from .argsme import (
     ArgsMeDocs, ArgsMeDoc, ArgsMeStance, ArgsMeMode, ArgsMeSourceDomain,
     ArgsMePremise, ArgsMePremiseAnnotation, ArgsMeAspect
 )
+
+from .extracted_cc import ExctractedCCDoc, ExctractedCCDocs, ExctractedCCQuery, ExctractedCCQueries
\ No newline at end of file
