@@ -3,6 +3,7 @@ import codecs
 import tarfile
 import re
 import gzip
+from glob import glob as fnglob
 import xml.etree.ElementTree as ET
 from fnmatch import fnmatch
 from pathlib import Path
@@ -23,6 +24,12 @@ class TitleUrlTextDoc(NamedTuple):
     url: str
     text: str
 
+class TrecParsedDoc(NamedTuple):
+    doc_id: str
+    title: str
+    body: str
+    marked_up_doc: bytes
+
 class TrecQuery(NamedTuple):
     query_id: str
     title: str
@@ -51,7 +58,7 @@ class TrecPrel(NamedTuple):
 CONTENT_TAGS = 'TEXT HEADLINE TITLE HL HEAD TTL DD DATE LP LEADPARA'.split()
 
 class TrecDocs(BaseDocs):
-    def __init__(self, docs_dlc, encoding=None, path_globs=None, content_tags=CONTENT_TAGS, parser='BS4', namespace=None, lang=None, expected_file_count=None, docstore_size_hint=None, count_hint=None):
+    def __init__(self, docs_dlc, encoding=None, path_globs=None, content_tags=CONTENT_TAGS, parser='BS4', namespace=None, lang=None, expected_file_count=None, docstore_size_hint=None, count_hint=None, docstore_path=None):
         self._docs_dlc = docs_dlc
         self._encoding = encoding
         self._path_globs = path_globs
@@ -60,17 +67,20 @@ class TrecDocs(BaseDocs):
             'BS4': self._parser_bs,
             'text': self._parser_text,
             'tut': self._parser_tut,
+            'sax': self._parser_sax,
         }[parser]
         self._doc = {
             'BS4': TrecDoc,
             'text': GenericDoc,
             'tut': TitleUrlTextDoc,
+            'sax': TrecParsedDoc,
         }[parser]
         self._docs_namespace = namespace
         self._docs_lang = lang
         self._expected_file_count = expected_file_count
         self._docstore_size_hint = docstore_size_hint
         self._count_hint = count_hint
+        self._docstore_path = docstore_path
         if expected_file_count is not None:
             assert self._path_globs is not None, "expected_file_count only supported with path_globs"
 
@@ -83,7 +93,10 @@ class TrecDocs(BaseDocs):
             if self._path_globs:
                 file_count = 0
                 for glob in sorted(self._path_globs):
-                    for path in sorted(Path(self._docs_dlc.path()).glob(glob)):
+                    glob_path = str(Path(self._docs_dlc.path())/glob)
+                    # IMPORTANT: cannot use Path().glob() here because the recusive ** will not follow symlinks.
+                    # Need to use glob.glob instead with recursive=True flag.
+                    for path in sorted(fnglob(glob_path, recursive=True)):
                         file_count += 1
                         yield from self._docs_iter(path)
                 if self._expected_file_count is not None:
@@ -113,9 +126,15 @@ class TrecDocs(BaseDocs):
 
     def _docs_iter(self, path):
         if Path(path).is_file():
-            if str(path).endswith('.gz'):
+            path_suffix = Path(path).suffix.lower()
+            if path_suffix == '.gz':
                 with gzip.open(path, 'rb') as f:
                     yield from self._parser(f)
+            elif path_suffix in ['.z', '.0z', '.1z', '.2z']:
+                # unix "compress" command encoding
+                unlzw3 = ir_datasets.lazy_libs.unlzw3()
+                with io.BytesIO(unlzw3.unlzw(path)) as f:
+                    yield from self._parser(f)
             else:
                 with path.open('rb') as f:
                     yield from self._parser(f)
@@ -190,12 +209,35 @@ class TrecDocs(BaseDocs):
                 if line.startswith('<TEXT>'):
                     in_tag = True
 
+    def _parser_sax(self, stream):
+        field_defs = []
+        field_defs.append({'docno'})
+        field_defs.append({'headline', 'title', 'h3', 'h4'})
+        field_defs.append({c.lower() for c in CONTENT_TAGS} - field_defs[-1])
+        buffer = bytearray()
+        while True:
+            if b'\n</DOC>' not in buffer:
+                chunk = stream.read1()
+                if chunk == b'':
+                    break
+                buffer.extend(chunk)
+            else:
+                idx = buffer.index(b'\n</DOC>')
+                full_doc = bytes(buffer[:idx+7])
+                doc_id, title, body = ir_datasets.util.html_parsing.sax_html_parser(full_doc, force_encoding=self._encoding or 'utf8', fields=field_defs)
+                yield TrecParsedDoc(doc_id, title, body, full_doc.strip())
+                del buffer[:idx+7]
+
     def docs_cls(self):
         return self._doc
 
     def docs_store(self, field='doc_id'):
+        if self._docstore_path is not None:
+            ds_path = self._docstore_path
+        else:
+            ds_path = f'{self.docs_path(force=False)}.pklz4'
         return PickleLz4FullStore(
-            path=f'{self.docs_path(force=False)}.pklz4',
+            path=ds_path,
             init_iter_fn=self.docs_iter,
             data_cls=self.docs_cls(),
             lookup_field=field,
@@ -349,23 +391,37 @@ class TrecColonQueries(BaseQueries):
 
 
 class TrecQrels(BaseQrels):
-    def __init__(self, qrels_dlc, qrels_defs):
+    def __init__(self, qrels_dlc, qrels_defs, format_3col=False):
         self._qrels_dlc = qrels_dlc
         self._qrels_defs = qrels_defs
+        self._format_3col = format_3col
 
     def qrels_path(self):
         return self._qrels_dlc.path()
 
     def qrels_iter(self):
-        with self._qrels_dlc.stream() as f:
+        if isinstance(self._qrels_dlc, list):
+            for dlc in self._qrels_dlc:
+                yield from self._qrels_internal_iter(dlc)
+        else:
+            yield from self._qrels_internal_iter(self._qrels_dlc)
+
+    def _qrels_internal_iter(self, dlc):
+        with dlc.stream() as f:
             f = codecs.getreader('utf8')(f)
             for line in f:
                 if line == '\n':
                     continue # ignore blank lines
                 cols = line.rstrip().split()
-                if len(cols) != 4:
-                    raise RuntimeError(f'expected 4 columns, got {len(cols)}')
-                qid, it, did, score = cols
+                if self._format_3col:
+                    if len(cols) != 3:
+                        raise RuntimeError(f'expected 3 columns, got {len(cols)}')
+                    qid, did, score = cols
+                    it = 'Q0'
+                else:
+                    if len(cols) != 4:
+                        raise RuntimeError(f'expected 4 columns, got {len(cols)}')
+                    qid, it, did, score = cols
                 yield TrecQrel(qid, did, int(score), it)
 
     def qrels_cls(self):
@@ -393,8 +449,9 @@ class TrecPrels(TrecQrels):
 
 
 class TrecScoredDocs(BaseScoredDocs):
-    def __init__(self, scoreddocs_dlc):
+    def __init__(self, scoreddocs_dlc, negate_score=False):
         self._scoreddocs_dlc = scoreddocs_dlc
+        self._negate_score = negate_score
 
     def scoreddocs_path(self):
         return self._scoreddocs_dlc.path()
@@ -405,7 +462,12 @@ class TrecScoredDocs(BaseScoredDocs):
             for line in f:
                 cols = line.rstrip().split()
                 if len(cols) == 6:
-                    qid, _, did, _, score, _ = cols
+                    qid, _, did, _, score, _ = cols # TREC-style (qid iteration did score rank score runtag
                 elif len(cols) == 2:
-                    qid, did, score = *cols, '0'
-                yield GenericScoredDoc(qid, did, float(score))
+                    qid, did, score = *cols, '0' # MS MARCO-style (qid did -- only)
+                elif len(cols) == 3:
+                    qid, did, score = cols # MMARCO-style (qid did score)
+                score = float(score)
+                if self._negate_score:
+                    score = -score
+                yield GenericScoredDoc(qid, did, score)
